# encoding: utf-8

# Création d'une image unie

def creer_image_gris (largeur, hauteur, maximum):
    return {
            'format': 'gris',
            'largeur': largeur,
            'hauteur': hauteur,
            'maximum': maximum,
            'donnees': bytearray(largeur * hauteur)
        }

def lire_pixel_gris (image, x, y):
    position = image['largeur'] * y + x
    return image['donnees'][position]

def ecrire_pixel_gris (image, x, y, valeur):
    position = image['largeur'] * y + x
    image['donnees'][position] = valeur


def creer_image_couleur (largeur, hauteur, maximum):
    return {
            'format': 'couleur',
            'largeur': largeur,
            'hauteur': hauteur,
            'maximum': maximum,
            'donnees': bytearray(largeur * hauteur * 3)
        }

def lire_pixel_couleur (image, x, y):
    position = (image['largeur'] * y + x) * 3
    rouge = image['donnees'][position]
    vert = image['donnees'][position + 1]
    bleu = image['donnees'][position + 2]
    return rouge, vert, bleu

def ecrire_pixel_couleur (image, x, y, rouge, vert, bleu):
    position = (image['largeur'] * y + x) * 3
    image['donnees'][position] = rouge
    image['donnees'][position + 1] = vert
    image['donnees'][position + 2] = bleu


# Écriture d'une image en format PNM

def ecrire_pbm_binaire (image, fichier):
    sortie = open(fichier, "wb")
    en_tete = "P3 %d %d\n" % (image['largeur'], image['hauteur'])
    sortie.write(en_tete.encode("ascii"))
    sortie.write(image['donnees'])

def ecrire_pgm_binaire (image, fichier):
    sortie = open(fichier, "wb")
    en_tete = "P5 %d %d %d\n" % (image['largeur'], image['hauteur'], image['maximum'])
    sortie.write(en_tete.encode("ascii"))
    sortie.write(image['donnees'])

def ecrire_ppm_binaire (image, fichier):
    sortie = open(fichier, "wb")
    en_tete = "P6 %d %d %d\n" % (image['largeur'], image['hauteur'], image['maximum'])
    sortie.write(en_tete.encode("ascii"))
    sortie.write(image['donnees'])


# Lecture du prochain "mot" sur un fichier PNM
#
# le mot est composé de tous les octets qui suivent jusqu'au prochain octet
# correspondant à un caractère d'espacement, lequel n'est pas conservé. Selon
# la spécification du format PNM, un caractère '#' introduit un commentaire,
# il est ignoré ainsi que tout ce qui le suit jusqu'au prochain retour à la
# ligne.

def lire_mot_pnm (entree):
    mot = b""
    while True:
        octet = entree.read(1)

        # Test de fin de fichier

        if len(octet) == 0:
            return mot

        # Test de fin de mot

        if octet in (b' ', b'\r', b'\n', b'\t'):
            if len(mot) != 0:
                return mot

        # Traitement des commentaires

        if octet == b'#':
            entree.readline()

        else:
            mot += octet

# Lecture d'un fichier image PNM
#
# Le format est déduit du premier mot du fichier.

def lire_pnm (fichier):
    entree = open(fichier, "rb")
    forme = lire_mot_pnm(entree)
    if forme == b'P3':
        return lire_ppm_ascii(entree)
    elif forme == b'P4':
        return lire_pbm_binaire(entree)
    elif forme == b'P5':
        return lire_pgm_binaire(entree)
    elif forme == b'P6':
        return lire_ppm_binaire(entree)
    else:
        raise ValueError("Format inconnu: %s" % forme.encode('ASCII'))

# Les fonctions suivantes servent à décoder un fichier image dont on connaît
# le format: on suppose que le premier mot a déjà été lu et on lit la suite.

# Format PBM binaire

def lire_pbm_binaire (entree):
    largeur = int(lire_mot_pnm(entree))
    hauteur = int(lire_mot_pnm(entree))
    ligne = (largeur + 7) // 8
    donnees = bytearray(entree.read(ligne * hauteur))
    return {
            'format': 'bitmap',
            'largeur': largeur,
            'hauteur': hauteur,
            'ligne': ligne,
            'donnees': donnees
        }

# Format PGM binaire

def lire_pgm_binaire (entree):
    largeur = int(lire_mot_pnm(entree))
    hauteur = int(lire_mot_pnm(entree))
    maximum = int(lire_mot_pnm(entree))
    if maximum > 255:
        raise ValueError("Intervalle trop grand (%d)" % maximum)
    donnees = bytearray(entree.read(largeur * hauteur))
    return {
            'format': 'gris',
            'largeur': largeur,
            'hauteur': hauteur,
            'maximum': maximum,
            'donnees': donnees
        }

# Format PPM binaire

def lire_ppm_binaire (entree):
    largeur = int(lire_mot_pnm(entree))
    hauteur = int(lire_mot_pnm(entree))
    maximum = int(lire_mot_pnm(entree))
    if maximum > 255:
        raise ValueError("Intervalle trop grand (%d)" % maximum)
    donnees = bytearray(entree.read(largeur * hauteur * 3))
    return {
            'format': 'couleur',
            'largeur': largeur,
            'hauteur': hauteur,
            'maximum': maximum,
            'donnees': donnees
        }

# Format PPM ascii

def lire_ppm_ascii (entree):
    largeur = int(lire_mot_pnm(entree))
    hauteur = int(lire_mot_pnm(entree))
    maximum = int(lire_mot_pnm(entree))
    if maximum > 255:
        raise ValueError("Intervalle trop grand (%d)" % maximum)
    nombre = largeur * hauteur * 3
    donnees = bytearray(nombre)
    for i in range(nombre):
        mot = lire_mot_pnm(entree)
        donnees[i] = int(mot)
    return {
            'format': 'couleur',
            'largeur': largeur,
            'hauteur': hauteur,
            'maximum': maximum,
            'donnees': donnees
        }


# Conversion de couleur vers gris

def couleurs_vers_gris (entree):
    if entree['format'] != 'couleur':
        raise ValueError("l'image n'est pas en couleur")
    largeur = entree['largeur']
    hauteur = entree['hauteur']
    sortie = creer_image_gris(largeur, hauteur, entree['maximum'])
    for x in range(largeur):
        for y in range(hauteur):
            r, v, b = lire_pixel_couleur(entree, x, y)
            teinte = (r + v + b) // 3
            ecrire_pixel_gris(sortie, x, y, teinte)
    return sortie

# Zoom d'un facteur arbitraire

def zoom_gris (entree, facteur):
    if entree['format'] != 'gris':
        raise ValueError("l'image n'est pas en gris")
    largeur = int(entree['largeur'] * facteur)
    hauteur = int(entree['hauteur'] * facteur)
    sortie = creer_image_gris(largeur, hauteur, entree['maximum'])
    for x in range(largeur):
        for y in range(hauteur):
            teinte = lire_pixel_gris(entree, int(x/facteur), int(y/facteur))
            ecrire_pixel_gris(sortie, x, y, teinte)
    return sortie

def test (largeur, hauteur):
    image = creer_image_gris(largeur, hauteur, 255)
    for x in range(largeur):
        for y in range(hauteur):
            teinte = ((x*x + y*y) // 20) % 256
            ecrire_pixel_gris(image, x, y, teinte)
    ecrire_pgm_binaire(image, "sortie.pgm")

if __name__ == '__main__':
    i1 = lire_pnm("pinklady.ppm")
    i2 = couleurs_vers_gris(i1)
    i3 = zoom_gris(i2, 10)
    ecrire_pgm_binaire(i3, "sortie.pgm")
